@@ -927,7 +927,7 @@ static void set_lib_and_class_name(char *mangled, char **out_class, char **out_l
 
 	*out_class = object;
 	if (!is_java_lang || startswith(object, "java.lang")) {
-		*out_lib = strdup(object);
+		*out_lib = rz_str_dup(object);
 	} else {
 		*out_lib = rz_str_newf("java.lang.%s", object);
 	}
@@ -1202,7 +1202,7 @@ static RzBinSection *section_new(const char *name, ut32 perm, ut32 size, ut64 pa
 	if (!section) {
 		return NULL;
 	}
-	section->name = strdup(name);
+	section->name = rz_str_dup(name);
 	section->paddr = paddr;
 	section->vaddr = vaddr;
 	section->size = section->vsize = size;
@@ -1688,12 +1688,12 @@ RZ_API RZ_OWN RzBinAddr *rz_bin_dex_resolve_symbol(RZ_NONNULL RzBinDex *dex, RzB
 	return ret;
 }
 
-static RzList /*<RzBinAddr *>*/ *dex_resolve_entrypoints_in_class(RzBinDex *dex, DexClassDef *class_def) {
+static RzPVector /*<RzBinAddr *>*/ *dex_resolve_entrypoints_in_class(RzBinDex *dex, DexClassDef *class_def) {
 	RzListIter *it;
 	DexEncodedMethod *encoded_method = NULL;
-	RzList *entrypoints = NULL;
+	RzPVector *entrypoints = NULL;
 
-	entrypoints = rz_list_newf((RzListFree)free);
+	entrypoints = rz_pvector_new((RzPVectorFree)free);
 	if (!entrypoints) {
 		return NULL;
 	}
@@ -1732,7 +1732,7 @@ static RzList /*<RzBinAddr *>*/ *dex_resolve_entrypoints_in_class(RzBinDex *dex,
 			entrypoint->vaddr = encoded_method->code_offset;
 			entrypoint->paddr = 0;
 		}
-		if (entrypoint && !rz_list_append(entrypoints, entrypoint)) {
+		if (entrypoint && !rz_pvector_push(entrypoints, entrypoint)) {
 			free(entrypoint);
 		}
 	}
@@ -1776,7 +1776,7 @@ static RzList /*<RzBinAddr *>*/ *dex_resolve_entrypoints_in_class(RzBinDex *dex,
 			entrypoint->vaddr = encoded_method->code_offset;
 			entrypoint->paddr = 0;
 		}
-		if (entrypoint && !rz_list_append(entrypoints, entrypoint)) {
+		if (entrypoint && !rz_pvector_push(entrypoints, entrypoint)) {
 			free(entrypoint);
 		}
 	}
@@ -1785,27 +1785,26 @@ static RzList /*<RzBinAddr *>*/ *dex_resolve_entrypoints_in_class(RzBinDex *dex,
 }
 
 /**
- * \brief Returns a RzList<RzBinAddr*> containing the dex entripoints
+ * \brief Returns a RzPVector<RzBinAddr*> containing the dex entripoints
  */
-RZ_API RZ_OWN RzList /*<RzBinAddr *>*/ *rz_bin_dex_entrypoints(RZ_NONNULL RzBinDex *dex) {
+RZ_API RZ_OWN RzPVector /*<RzBinAddr *>*/ *rz_bin_dex_entrypoints(RZ_NONNULL RzBinDex *dex) {
 	rz_return_val_if_fail(dex, NULL);
 
 	DexClassDef *class_def;
-	RzList *list = NULL;
-	RzList *entrypoints = NULL;
+	RzPVector *entrypoints = NULL, *vec = NULL;
 	void **it;
 
-	entrypoints = rz_list_newf((RzListFree)free);
+	entrypoints = rz_pvector_new((RzPVectorFree)free);
 	if (!entrypoints) {
 		return NULL;
 	}
 
 	rz_pvector_foreach (dex->class_defs, it) {
 		class_def = (DexClassDef *)*it;
-		list = dex_resolve_entrypoints_in_class(dex, class_def);
-		if (list) {
-			rz_list_join(entrypoints, list);
-			rz_list_free(list);
+		vec = dex_resolve_entrypoints_in_class(dex, class_def);
+		if (vec) {
+			rz_pvector_join(entrypoints, vec);
+			rz_pvector_free(vec);
 		}
 	}
 
@@ -2016,19 +2015,19 @@ RZ_API RZ_OWN char *rz_bin_dex_version(RZ_NONNULL RzBinDex *dex) {
 	// https://cs.android.com/android/platform/superproject/+/master:dalvik/dx/src/com/android/dex/DexFormat.java;l=55;bpv=1;bpt=0
 	// https://developer.android.com/studio/releases/platforms
 	if (startswith((char *)dex->version, "009")) {
-		return strdup("Android M3 release (Nov-Dec 2007)");
+		return rz_str_dup("Android M3 release (Nov-Dec 2007)");
 	} else if (startswith((char *)dex->version, "013")) {
-		return strdup("Android M5 release (Feb-Mar 2008)");
+		return rz_str_dup("Android M5 release (Feb-Mar 2008)");
 	} else if (startswith((char *)dex->version, "035")) {
-		return strdup("Android 3.2 (API level 13 and earlier)");
+		return rz_str_dup("Android 3.2 (API level 13 and earlier)");
 	} else if (startswith((char *)dex->version, "037")) {
-		return strdup("Android 7 (API level 24 and earlier)");
+		return rz_str_dup("Android 7 (API level 24 and earlier)");
 	} else if (startswith((char *)dex->version, "038")) {
-		return strdup("Android 8 (API level 26 and earlier)");
+		return rz_str_dup("Android 8 (API level 26 and earlier)");
 	} else if (startswith((char *)dex->version, "039")) {
-		return strdup("Android 9 (API level 28 and earlier)");
+		return rz_str_dup("Android 9 (API level 28 and earlier)");
 	} else if (startswith((char *)dex->version, "040")) {
-		return strdup("Android 10+ (Aug 2019)");
+		return rz_str_dup("Android 10+ (Aug 2019)");
 	}
 	return NULL;
 }
