@@ -29,18 +29,18 @@ static RzBinInfo *info(RzBinFile *bf) {
 	if (!(ret = RZ_NEW0(RzBinInfo))) {
 		return NULL;
 	}
-	ret->file = strdup(bf->file);
-	ret->type = strdup("Sound File Data");
-	ret->machine = strdup("SPC700");
-	ret->os = strdup("spc700");
-	ret->arch = strdup("spc700");
+	ret->file = rz_str_dup(bf->file);
+	ret->type = rz_str_dup("Sound File Data");
+	ret->machine = rz_str_dup("SPC700");
+	ret->os = rz_str_dup("spc700");
+	ret->arch = rz_str_dup("spc700");
 	ret->bits = 16;
 	ret->has_va = 1;
 	return ret;
 }
 
-static RzList /*<RzBinSection *>*/ *sections(RzBinFile *bf) {
-	RzList *ret = NULL;
+static RzPVector /*<RzBinSection *>*/ *sections(RzBinFile *bf) {
+	RzPVector *ret = NULL;
 	RzBinSection *ptr = NULL;
 	spc_hdr spchdr;
 	memset(&spchdr, 0, SPC_HDR_SIZE);
@@ -49,31 +49,31 @@ static RzList /*<RzBinSection *>*/ *sections(RzBinFile *bf) {
 		eprintf("Truncated Header\n");
 		return NULL;
 	}
-	if (!(ret = rz_list_new())) {
+	if (!(ret = rz_pvector_new(NULL))) {
 		return NULL;
 	}
 	if (!(ptr = RZ_NEW0(RzBinSection))) {
-		rz_list_free(ret);
+		rz_pvector_free(ret);
 		return NULL;
 	}
-	ptr->name = strdup("RAM");
+	ptr->name = rz_str_dup("RAM");
 	ptr->paddr = RAM_START_ADDRESS;
 	ptr->size = RAM_SIZE;
 	ptr->vaddr = 0x0;
 	ptr->vsize = RAM_SIZE;
 	ptr->perm = RZ_PERM_R;
-	rz_list_append(ret, ptr);
+	rz_pvector_push(ret, ptr);
 	return ret;
 }
 
-static RzList /*<RzBinAddr *>*/ *entries(RzBinFile *bf) {
-	RzList *ret = rz_list_newf(free);
+static RzPVector /*<RzBinAddr *>*/ *entries(RzBinFile *bf) {
+	RzPVector *ret = rz_pvector_new(free);
 	if (ret) {
 		RzBinAddr *ptr = RZ_NEW0(RzBinAddr);
 		if (ptr) {
 			ptr->paddr = RAM_START_ADDRESS;
 			ptr->vaddr = 0;
-			rz_list_append(ret, ptr);
+			rz_pvector_push(ret, ptr);
 		}
 	}
 	return ret;
