@@ -6,6 +6,44 @@
 
 #include <rz_util.h>
 
+RZ_IPI bool tpi_stream_parse(RzPdb *pdb, RzPdbMsfStream *stream);
+RZ_IPI RzPdbTpiType *simple_type_parse(RzPdbTpiStream *stream, ut32 idx);
+RZ_IPI void tpi_stream_free(RzPdbTpiStream *stream);
+
+/// enumeration for virtual shape table entries
+typedef enum {
+	VTS_near = 0x00,
+	VTS_far = 0x01,
+	VTS_thin = 0x02,
+	VTS_outer = 0x03,
+	VTS_meta = 0x04,
+	VTS_near32 = 0x05,
+	VTS_far32 = 0x06,
+	VTS_unused = 0x07
+} TpiCV_VTS_desc;
+
+/// enumeration for LF_LABEL address modes
+typedef enum {
+	LABEL_NEAR = 0, // near return
+	LABEL_FAR = 4 // far return
+} TpiCV_LABEL_TYPE;
+
+/// enumeration for HFA kinds
+typedef enum {
+	CV_HFA_none = 0,
+	CV_HFA_float = 1,
+	CV_HFA_double = 2,
+	CV_HFA_other = 3
+} TpiCV_HFA;
+
+/// enumeration for MoCOM UDT kinds
+typedef enum CV_MOCOM_UDT_e {
+	CV_MOCOM_UDT_none = 0,
+	CV_MOCOM_UDT_ref = 1,
+	CV_MOCOM_UDT_value = 2,
+	CV_MOCOM_UDT_interface = 3
+} TpiCV_MOCOM_UDT;
+
 typedef enum {
 	DIRECT = 0, // Not a pointer
 	NEAR_POINTER = 1, // Near pointer
@@ -95,6 +133,7 @@ typedef enum {
 	LF_ENDPRECOMP = 0x0014, // type record describing end of precompiled types that can be
 	LF_OEM_16t = 0x0015, // type record for OEM definable type strings
 	LF_TYPESERVER_ST = 0x0016, // type record describing using of a type server
+
 	LF_SKIP_16t = 0x0200,
 	LF_ARGLIST_16t = 0x0201,
 	LF_DEFARG_16t = 0x0202,
@@ -108,6 +147,7 @@ typedef enum {
 	LF_DIMVARU_16t = 0x020A, // type record for dimensioned array with variable bounds
 	LF_DIMVARLU_16t = 0x020B, // type record for dimensioned array with variable bounds
 	LF_REFSYM = 0x020C, // type record for referenced symbol
+
 	LF_BCLASS_16t = 0x0400, // subfield record for base class field
 	LF_VBCLASS_16t = 0x0401, // subfield record for direct and indirect virtual base class field
 	LF_IVBCLASS_16t = 0x0402,
@@ -122,6 +162,7 @@ typedef enum {
 	LF_FRIENDCLS_16t = 0x040B, // subfield record for friend class
 	LF_ONEMETHOD_16t = 0x040C, // subfield record for nonoverloaded method
 	LF_VFUNCOFF_16t = 0x040D, // subfield record for virtual function table pointer with offset
+
 	LF_TI16_MAX = 0x1000,
 	LF_MODIFIER = 0x1001, // type record for a generalized built-in type modifier
 	LF_POINTER = 0x1002,
@@ -140,6 +181,7 @@ typedef enum {
 	LF_OEM = 0x100F, // type record for OEM definable type strings
 	LF_ALIAS_ST = 0x1010,
 	LF_OEM2 = 0x1011, // type record for OEM definable type strings
+
 	LF_SKIP = 0x1200,
 	LF_ARGLIST = 0x1201,
 	LF_DEFARG_ST = 0x1202,
@@ -151,6 +193,7 @@ typedef enum {
 	LF_DIMCONLU = 0x1208, // type record for dimensioned array with constant bounds
 	LF_DIMVARU = 0x1209, // type record for dimensioned array with variable bounds
 	LF_DIMVARLU = 0x120A, // type record for dimensioned array with variable bounds
+
 	LF_BCLASS = 0x1400, // subfield record for base class field
 	LF_VBCLASS = 0x1401, // subfield record for direct and indirect virtual base class field
 	LF_IVBCLASS = 0x1402,
@@ -167,6 +210,7 @@ typedef enum {
 	LF_NESTTYPEEX_ST = 0x140D, // type record for nested (scoped) type definition, with attributes
 	LF_MEMBERMODIFY_ST = 0x140E, // type record for modifications to members
 	LF_MANAGED_ST = 0x140F,
+
 	LF_ST_MAX = 0x1500,
 	LF_TYPESERVER = 0x1501, // type record describing using of a type server
 	LF_ENUMERATE = 0x1502, // subfield record for enumerate
@@ -189,6 +233,18 @@ typedef enum {
 	LF_MEMBERMODIFY = 0x1513, // type record for modifications to members
 	LF_MANAGED = 0x1514,
 	LF_TYPESERVER2 = 0x1515, // type record describing using of a type server with v7 (GUID) signatures
+	LF_STRIDED_ARRAY = 0x1516, // same as LF_ARRAY, but with stride between adjacent elements
+	LF_HLSL = 0x1517,
+	LF_MODIFIER_EX = 0x1518,
+	LF_INTERFACE = 0x1519,
+	LF_BINTERFACE = 0x151a,
+	LF_VECTOR = 0x151b,
+	LF_MATRIX = 0x151c,
+	LF_VFTABLE = 0x151d, // a virtual function table
+	LF_ENDOFLEAFRECORD = LF_VFTABLE,
+	LF_TYPE_LAST, // one greater than the last type record
+	LF_TYPE_MAX = LF_TYPE_LAST - 1,
+
 	LF_FUNCTION_ID = 0x1601,
 	LF_MEMBER_FUNCTION_ID = 0x1602,
 	LF_BUILD_INFO = 0x1603,
@@ -198,6 +254,10 @@ typedef enum {
 	LF_USER_DEFINED_TYPE_MODULE_SOURCE_AND_LINE = 0x1607,
 	LF_CLASS_19 = 0x1608,
 	LF_STRUCTURE_19 = 0x1609,
+	LF_UNION_19 = 0x160A,
+	LF_INTERFACE_19 = 0x160B,
+	LF_ID_LAST, // one greater than the last ID record
+	LF_ID_MAX = LF_ID_LAST - 1,
 
 	/**     the following are numeric leaves.  They are used to indicate the
 	 *      size of the following variable length data.  When the numeric
@@ -205,6 +265,7 @@ typedef enum {
 	 *      directly.  If the data is more the 0x8000 or is a negative value,
 	 *      then the data is preceeded by the proper index.
 	 */
+	LF_NUMERIC = 0x8000,
 	LF_CHAR = 0x8000, // signed character leaf
 	LF_SHORT = 0x8001, // signed short leaf
 	LF_USHORT = 0x8002, // unsigned short leaf
@@ -227,6 +288,8 @@ typedef enum {
 	LF_DECIMAL = 0x8019,
 	LF_DATE = 0x801A,
 	LF_UTF8STRING = 0x801B,
+	LF_REAL16 = 0x801c,
+
 	LF_PAD0 = 0x00F0,
 	LF_PAD1 = 0x00F1,
 	LF_PAD2 = 0x00F2,
@@ -247,23 +310,21 @@ typedef enum {
 	LF_MAX = 0xFFFF
 } TpiLeafType;
 
-typedef union {
-	struct {
-		ut16 packed : 1; // true if structure is packed
-		ut16 ctor : 1; // true if constructors or destructors present
-		ut16 ovlops : 1; // true if overloaded operators present
-		ut16 isnested : 1; // true if this is a nested class
-		ut16 cnested : 1; // true if this class contains nested types
-		ut16 opassign : 1; // true if overloaded assignment (=)
-		ut16 opcast : 1; // true if casting methods
-		ut16 fwdref : 1; // true if forward reference (incomplete defn)
-		ut16 scoped : 1; // scoped definition
-		ut16 hasuniquename : 1; // true if there is a decorated name following the regular name
-		ut16 sealed : 1; // true if class cannot be used as a base class
-		ut16 hfa : 2; // CV_HFA_e
-		ut16 intrinsic : 1; // true if class is an intrinsic type (e.g. __m128d)
-		ut16 mocom : 2; // CV_MOCOM_UDT_e
-	} bits;
+typedef struct {
+	ut16 packed : 1; // true if structure is packed
+	ut16 ctor : 1; // true if constructors or destructors present
+	ut16 ovlops : 1; // true if overloaded operators present
+	ut16 isnested : 1; // true if this is a nested class
+	ut16 cnested : 1; // true if this class contains nested types
+	ut16 opassign : 1; // true if overloaded assignment (=)
+	ut16 opcast : 1; // true if casting methods
+	ut16 fwdref : 1; // true if forward reference (incomplete defn)
+	ut16 scoped : 1; // scoped definition
+	ut16 has_uniquename : 1; // true if there is a decorated name following the regular name
+	ut16 sealed : 1; // true if class cannot be used as a base class
+	ut16 hfa : 2; // CV_HFA_e
+	ut16 intrinsic : 1; // true if class is an intrinsic type (e.g. __m128d)
+	ut16 mocom : 2; // CV_MOCOM_UDT_e
 } TpiCVProperty;
 
 typedef enum {
@@ -284,31 +345,27 @@ typedef enum {
 	AccessMax
 } TpiCVAccess;
 
-typedef union {
-	struct {
-		ut16 access : 2; // access protection CV_access_t
-		ut16 mprop : 3; // method properties CV_methodprop_t
-		ut16 pseudo : 1; // compiler generated fcn and does not exist
-		ut16 noinherit : 1; // true if class cannot be inherited
-		ut16 noconstruct : 1; // true if class cannot be constructed
-		ut16 compgenx : 1; // compiler generated fcn and does exist
-		ut16 sealed : 1; // true if method cannot be overridden
-		ut16 unused : 6; // unused
-	} bits;
+typedef struct {
+	ut16 access : 2; // access protection CV_access_t
+	ut16 mprop : 3; // method properties CV_methodprop_t
+	ut16 pseudo : 1; // compiler generated fcn and does not exist
+	ut16 noinherit : 1; // true if class cannot be inherited
+	ut16 noconstruct : 1; // true if class cannot be constructed
+	ut16 compgenx : 1; // compiler generated fcn and does exist
+	ut16 sealed : 1; // true if method cannot be overridden
+	ut16 unused : 6; // unused
 } TpiCVFldattr;
 
-typedef union {
-	struct cv_funcattr {
-		unsigned char cxxreturnudt : 1; // true if C++ style ReturnUDT
-		unsigned char ctor : 1; // true if func is an instance constructor
-		unsigned char ctorvbase : 1; // true if func is an instance constructor of a class with virtual bases
-		unsigned char unused : 5; // unused
-	} bits;
+typedef struct cv_funcattr {
+	RzPdbTpiCallingConvention calling_convention;
+	ut8 cxxreturnudt : 1; // true if C++ style ReturnUDT
+	ut8 ctor : 1; // true if func is an instance constructor
+	ut8 ctorvbase : 1; // true if func is an instance constructor of a class with virtual bases
+	ut8 unused : 5; // unused
 } TpiCVFuncattr;
 
 typedef struct {
 	ut32 return_type;
-	RzPdbTpiCallingConvention call_conv;
 	TpiCVFuncattr func_attr;
 	ut16 parm_count;
 	ut32 arg_list;
@@ -319,7 +376,6 @@ typedef struct {
 	ut32 return_type;
 	ut32 class_type;
 	ut32 this_type;
-	RzPdbTpiCallingConvention call_conv; // 1 byte
 	TpiCVFuncattr func_attr;
 	ut16 parm_count;
 	ut32 arglist;
@@ -333,13 +389,11 @@ typedef struct {
 	ut8 pad;
 } Tpi_LF_Arglist;
 
-typedef union {
-	struct {
-		ut16 const_ : 1;
-		ut16 volatile_ : 1;
-		ut16 unaligned : 1;
-		ut16 unused : 13;
-	} bits;
+typedef struct {
+	ut16 const_ : 1;
+	ut16 volatile_ : 1;
+	ut16 unaligned : 1;
+	ut16 unused : 13;
 } TpiCVModifier;
 
 typedef struct {
@@ -355,7 +409,7 @@ typedef enum {
 	PTR_MODE_PMFUNC = 0x00000003, // pointer to member function
 	PTR_MODE_RVREF = 0x00000004, // r-value reference
 	PTR_MODE_RESERVED = 0x00000005, // first unused pointer mode
-	ModeMax
+	PTR_MODE_Max
 } TpiCVPtrMode;
 
 typedef enum {
@@ -385,92 +439,88 @@ typedef enum {
 	PTR_FAR32 = 0x0000000B,
 	PTR_64 = 0x0000000C,
 	PTR_UNUSEDPTR = 0x0000000D,
-	TypeMax
+	PTR_Max
 } TpiCVType;
 
-typedef union {
-	struct {
-		ut32 ptrtype : 5; // ordinal specifying pointer type
-		ut32 ptrmode : 3; // ordinal specifying pointer mode
-		ut32 flat32 : 1; // true if 0:32 pointer
-		ut32 volatile_ : 1; // TRUE if volatile pointer
-		ut32 const_ : 1; // TRUE if const pointer
-		ut32 unaligned : 1; // TRUE if unaligned pointer
-		ut32 restrict_ : 1; // TRUE if restricted pointer (allow agressive opts)
-		ut32 size : 6; // size of pointer (in bytes)
-		ut32 mocom : 1; // TRUE if it is a MoCOM pointer (^ or %)
-		ut32 lref : 1; // TRUE if it is this pointer of member function with & ref-qualifier
-		ut32 rref : 1; // TRUE if it is this pointer of member function with && ref-qualifier
-		ut32 unused : 10; // pad out to 32-bits for following cv_typ_t's
-	} bits;
+typedef struct {
+	ut32 ptrtype : 5; // ordinal specifying pointer type
+	ut32 ptrmode : 3; // ordinal specifying pointer mode
+	ut32 flat32 : 1; // true if 0:32 pointer
+	ut32 volatile_ : 1; // TRUE if volatile pointer
+	ut32 const_ : 1; // TRUE if const pointer
+	ut32 unaligned : 1; // TRUE if unaligned pointer
+	ut32 restrict_ : 1; // TRUE if restricted pointer (allow agressive opts)
+	ut32 size : 6; // size of pointer (in bytes)
+	ut32 mocom : 1; // TRUE if it is a MoCOM pointer (^ or %)
+	ut32 lref : 1; // TRUE if it is this pointer of member function with & ref-qualifier
+	ut32 rref : 1; // TRUE if it is this pointer of member function with && ref-qualifier
+	ut32 unused : 10; // pad out to 32-bits for following cv_typ_t's
 } TpiCVPointerAttr;
 
 typedef struct {
 	ut32 utype;
 	TpiCVPointerAttr ptr_attr;
-	union {
-		struct {
-			ut32 pmclass; // index of containing class for pointer to member
-			ut16 pmtype; // TpiCVPmType
-		} pmember;
-		struct {
-			ut32 index; // type index if PTR_BASE_TYPE
-		} pbase;
-	};
+	PDBTypeIndex containing_class;
 	ut8 pad;
 } Tpi_LF_Pointer;
 
 typedef struct {
-	char *name;
-	ut32 size;
-} Tpi_Type_String;
-
-typedef struct {
-	ut16 type_index;
-	void *data;
-	bool is_integer;
-} Tpi_Type_Numeric;
+	enum {
+		TpiVariant_U64,
+		TpiVariant_U32,
+		TpiVariant_U16,
+		TpiVariant_U8,
+		TpiVariant_I64,
+		TpiVariant_I32,
+		TpiVariant_I16,
+		TpiVariant_I8,
+	} tag;
+	union {
+		ut64 u64v;
+		ut32 u32v;
+		ut16 u16v;
+		ut8 u8v;
+		st64 i64v;
+		st32 i32v;
+		st16 i16v;
+		st8 i8v;
+	};
+} TpiVariant;
 
 typedef struct {
 	ut32 element_type;
 	ut32 index_type;
-	Tpi_Type_Numeric size;
-	Tpi_Type_String name;
+	ut32 stride;
+	RzVector /*<ut32>*/ dimensions;
 	ut8 pad;
 } Tpi_LF_Array;
 
-typedef struct {
-	ut16 count;
-	TpiCVProperty prop; // property attribute field
-	ut32 field_list; // type index of LF_FIELD descriptor list
-	ut32 derived; // type index of derived from list if not zero
-	ut32 vshape; // type index of vshape table for this class
-	Tpi_Type_Numeric size;
-	Tpi_Type_String name;
-	Tpi_Type_String mangled_name;
-	ut8 pad;
-} Tpi_LF_Structure, Tpi_LF_Class;
+typedef enum {
+	ClassKind_Class,
+	ClassKind_Struct,
+	ClassKind_Interface
+} ClassKind;
 
 typedef struct {
+	ut16 count;
+	ClassKind kind;
 	TpiCVProperty prop; // property attribute field
-	ut16 unknown;
 	ut32 field_list; // type index of LF_FIELD descriptor list
 	ut32 derived; // type index of derived from list if not zero
 	ut32 vshape; // type index of vshape table for this class
-	Tpi_Type_Numeric unknown1;
-	Tpi_Type_Numeric size;
-	Tpi_Type_String name;
-	Tpi_Type_String mangled_name;
+	ut64 size;
+	char *name;
+	char *mangled_name;
 	ut8 pad;
-} Tpi_LF_Structure_19, Tpi_LF_Class_19;
+} Tpi_LF_Structure, Tpi_LF_Class;
 
 typedef struct {
 	ut16 count;
 	TpiCVProperty prop;
 	ut32 field_list;
-	Tpi_Type_Numeric size;
-	Tpi_Type_String name;
-	Tpi_Type_String mangled_name;
+	ut64 size;
+	char *name;
+	char *mangled_name;
 	ut32 pad;
 } Tpi_LF_Union;
 
@@ -483,31 +533,47 @@ typedef struct {
 
 typedef struct {
 	ut16 count;
-	char *vt_descriptors;
+	RzVector /*<ut8>*/ descriptors;
 	ut8 pad;
 } Tpi_LF_Vtshape;
 
+/// type record for a virtual function table
+typedef struct {
+	ut16 leaf; // LF_VFTABLE
+	TpiCVType complete_class; // class/structure that owns the vftable
+	TpiCVType override_vftable; // vftable from which this vftable is derived
+	ut32 vfptr_offset; // offset of the vfptr to this table, relative to the start of the object layout.
+	RzPVector /*<char *>*/ method_names; // array of names.
+					     // The first is the name of the vtable.
+					     // The others are the names of the methods.
+} Tpi_LF_Vftable;
+
+// LF_LABEL
+typedef struct {
+	TpiCV_LABEL_TYPE mode;
+} Tpi_LF_Label;
+
 typedef struct {
 	ut16 count;
 	TpiCVProperty prop;
 	ut32 utype;
 	ut32 field_list;
-	Tpi_Type_String name;
-	Tpi_Type_String mangled_name;
+	char *name;
+	char *mangled_name;
 	ut8 pad;
 } Tpi_LF_Enum;
 
 typedef struct {
 	TpiCVFldattr fldattr;
-	Tpi_Type_Numeric enum_value;
-	Tpi_Type_String name;
+	TpiVariant value;
+	char *name;
 	ut8 pad;
 } Tpi_LF_Enumerate;
 
 typedef struct {
-	ut16 pad;
+	TpiCVFldattr fldattr;
 	ut32 index;
-	Tpi_Type_String name;
+	char *name;
 } Tpi_LF_NestType;
 
 typedef struct {
@@ -518,7 +584,7 @@ typedef struct {
 typedef struct {
 	ut16 count;
 	ut32 mlist;
-	Tpi_Type_String name;
+	char *name;
 	ut8 pad;
 } Tpi_LF_Method;
 
@@ -530,42 +596,43 @@ typedef struct {
 } Tpi_Type_MethodListMember;
 
 typedef struct {
-	RzList /*<Tpi_Type_MethodListMember *>*/ *members;
+	RzPVector /*<Tpi_Type_MethodListMember *>*/ members;
 } Tpi_LF_MethodList;
 
 typedef struct {
 	TpiCVFldattr fldattr;
-	ut32 index;
-	Tpi_Type_Numeric offset;
-	Tpi_Type_String name;
+	ut32 field_type;
+	ut64 offset;
+	char *name;
 	ut8 pad;
 } Tpi_LF_Member;
 
 typedef struct {
 	TpiCVFldattr fldattr;
-	ut32 index;
-	Tpi_Type_String name;
+	ut32 field_type;
+	char *name;
 	ut8 pad;
 } Tpi_LF_StaticMember;
 
 typedef struct {
-	TpiCVFldattr fldattr;
+	ut16 leaf;
+	ut16 pad0;
 	ut32 index;
-	ut8 pad;
 } Tpi_LF_Index;
 
 typedef struct {
 	TpiCVFldattr fldattr;
 	ut32 index;
 	ut32 offset_in_vtable;
-	Tpi_Type_String name;
+	char *name;
 	ut8 pad;
 } Tpi_LF_OneMethod;
 
 typedef struct {
+	ClassKind kind;
 	TpiCVFldattr fldattr;
 	ut32 index;
-	Tpi_Type_Numeric offset;
+	ut64 offset;
 	ut8 pad;
 } Tpi_LF_BClass;
 
@@ -573,12 +640,12 @@ typedef struct {
 	TpiCVFldattr fldattr;
 	ut32 direct_vbclass_idx;
 	ut32 vb_pointer_idx;
-	Tpi_Type_Numeric vb_pointer_offset;
-	Tpi_Type_Numeric vb_offset_from_vbtable;
+	ut64 vb_pointer_offset;
+	ut64 vb_offset_from_vbtable;
 } Tpi_LF_VBClass, Tpi_LF_IVBClass;
 
 typedef struct {
-	RzList /*<RzPdbTpiType *>*/ *substructs;
+	RzPVector /*<RzPdbTpiType *>*/ *substructs;
 } Tpi_LF_FieldList;
 
 typedef struct {
